# voicevox-client - supported devices

from typing import Dict


class SupportedDevices:
    """Supported devices
    
    Attributes
    ----------
    cpu : bool
        Check cpu support
    cuda : bool
        Check cuda support
    dml : bool
        Check directml support
    """
    def __init__(self, payload: Dict[str, bool]):
        self.__data = payload

    @property
    def cpu(self) -> bool:
        return self.__data["cpu"]

    @property
    def cuda(self) -> bool:
        return self.__data["cuda"]

    @property
    def dml(self) -> bool:
        return self.__data["dml"]



# voicevox - audio_query

from __future__ import annotations
from typing import Optional, List

from .types import AudioQueryType, AccentPhraseType, MoraType
from .http import HttpClient


class Mora:
    """Mora class

    Attributes
    ----------
    text: str
        Text to synthesize
    consonant: str
        Start of consonants
    consonant_length: int
        Pronounciation length of consonants
    vowel: str
        Start of vowels
    vowel_length: int
        Length of vowels
    pitch: int
        ピッチ //Pitch of the voice
    """

    def __init__(self, payload: MoraType):
        self.text: str = payload["text"]
        self.consonant: str = payload["consonant"]
        self.consonant_length: int = payload["consonant_length"]
        self.vowel: int = payload["vowel"]
        self.vowel_length: int = payload["vowel_length"]
        self.pitch: int = payload["pitch"]

    def to_dict(self) -> dict:
        return {
            "text": self.text,
            "consonant": self.consonant,
            "consonant_length": self.consonant_length,
            "vowel": self.vowel,
            "vowel_length": self.vowel_length,
            "pitch": self.pitch
        }


class AccentPhrase:
    """
    Accent phrase class

    Attributes
    ----------
    moras: List[Mora]
        モーラのリスト
    accent: int
        アクセント箇所
    pause_mora: Optional[Mora]
        後ろに無音を付けるかどうか
    is_interrogative: bool
        疑問系かどうか
    """

    def __init__(self, payload: AccentPhraseType):
        self.moras: list = [Mora(mora) for mora in payload["moras"]]
        self.accent: int = payload["accent"]
        if payload.get("pause_mora") is not None:
            self.pause_mora: Optional[Mora] = Mora(payload.get("pause_mora"))
        else:
            self.pause_mora: Optional[Mora] = None
        self.is_interrogative: bool = payload["is_interrogative"]

    def to_dict(self) -> AccentPhraseType:
        payload = {
            "moras": [mora.to_dict() for mora in self.moras],
            "accent": self.accent,
            "is_interrogative": self.is_interrogative
        }
        if self.pause_mora is not None:
            payload["pause_mora"] = self.pause_mora.to_dict()
        else:
            payload["pause_mora"] = None
        return payload


class AudioQuery:
    """Audio query

    Audio query to do synthesis.

    Attributes
    ----------
    accent_phrases: dict
        アクセント句のリスト
    speed_scale: float
        Speech speed
    pitch_scale: float
        Speech pitch
    intonation_scale: float
        Speech intonation
    volume_scale: float
        Speech volume
    pre_phoneme_length: float
        音声の前の無音時間
    post_phoneme_length: float
        音声の後の無音時間
    output_sampling_rate: int
        音声データの出力サンプリングレート
    output_stereo: bool
        音声データをステレオ出力するか否か
    kana: str
        [読み取り専用]AquesTalkライクな読み仮名。音声合成クエリとしては無視される
    """

    def __init__(
        self, http: HttpClient, payload: AudioQueryType
    ):
        self.http = http
        self.__data = payload

        self.accent_phrases: List[AccentPhrase] = [
            AccentPhrase(accent_phrase)
            for accent_phrase in payload["accent_phrases"]
        ]
        self.speed_scale: float = payload["speedScale"]
        self.pitch_scale: float = payload["pitchScale"]
        self.intonation_scale: float = payload["intonationScale"]
        self.volume_scale: float = payload["volumeScale"]
        self.pre_phoneme_length: float = payload["prePhonemeLength"]
        self.post_phoneme_length: float = payload["postPhonemeLength"]
        self.output_sampling_rate: int = payload["outputSamplingRate"]
        self.output_stereo: bool = payload["outputStereo"]

    @property
    def kana(self) -> str:
        return self.__data["kana"]

    def to_dict(self) -> AudioQueryType:
        return {
            "accent_phrases": [
                accent_phrase.to_dict()
                for accent_phrase in self.accent_phrases
            ],
            "speedScale": self.speed_scale,
            "pitchScale": self.pitch_scale,
            "intonationScale": self.intonation_scale,
            "volumeScale": self.volume_scale,
            "prePhonemeLength": self.pre_phoneme_length,
            "postPhonemeLength": self.post_phoneme_length,
            "outputSamplingRate": self.output_sampling_rate,
            "outputStereo": self.output_stereo,
            "kana": self.kana
        }

    async def synthesis(
        self, *, enable_interrogative_upspeak: bool = True,
        speaker: int, core_version: Optional[str] = None
    ) -> bytes:
        params = {
            "speaker": speaker,
            "enable_interrogative_upspeak": enable_interrogative_upspeak
        }
        if core_version is not None:
            params["core_version"] = core_version
        return await self.http.synthesis(params, self.to_dict())



# voicevox - speakers

from typing import List

from .types import SpeakerType, StyleType, SupportedFeatureType


class SupportedFeature:
    """Return supported feature info

    Attributes
    ----------
    permitted_synthesis_morphing: str
        return permitted_synthesis_morphing
    """

    def __init__(self, payload: SupportedFeatureType):
        self.permitted_synthesis_morphing: str = payload[
            "permitted_synthesis_morphing"
        ]


class Style:
    """Return style info

    Attributes
    ----------
    name: str
        style name
    id: int
        style id
    """

    def __init__(self, payload: StyleType):
        self.__data = payload

    @property
    def name(self) -> str:
        return self.__data["name"]

    @property
    def id(self) -> int:
        return self.__data["id"]


class Speaker:
    """Return speaker info

    Attributes
    ----------
    supported_features: SupportedFeature
        return supported_features
    name: str
        speaker name
    uuid: str
        speaker uuid
    version: str
        speaker version
    styles: List[Style]
        speaker styles
    """

    def __init__(self, payload: SpeakerType):
        self.supported_features: SupportedFeature = SupportedFeature(
            payload["supported_features"]
        )
        self.name: str = payload["name"]
        self.uuid: str = payload["speaker_uuid"]
        self.version: str = payload["version"]
        self.styles: List[Style] = [
            Style(style) for style in payload["styles"]
        ]



# voicevox - speaker_info

from typing import List

from .types.speaker_info import SpeakerInfoType, StyleInfoType


class StyleInfo:
    """Return style info

    Attributes
    ----------
    id: int
        style id
    icon: str
        base64 encoded icon
    portrait: str
        base64 encoded portrait image
    voice_samples: list[str]
        base64 encoded voice sample
    """

    def __init__(self, payload: StyleInfoType):
        self.__data = payload

    @property
    def id(self) -> int:
        return self.__data["id"]
    
    @property
    def icon(self) -> str:
        return self.__data["icon"]
    
    @property
    def portrait(self) -> str:
        return self.__data["portrait"]
    
    @property
    def voice_samples(self) -> list[str]:
        return self.__data["voice_samples"]
    

class SpeakerInfo:
    """Return speaker info

    Attributes
    ----------
    policy: str
        policy
    portrait: str
        base64 encoded portrait image
    style_infos: list[StyleInfo]
        list of Style informations
    """

    def __init__(self, payload: SpeakerInfoType):
        self.policy: str = payload["policy"]
        self.portrait: str = payload["portrait"]
        self.style_infos: List[StyleInfo] = [
            StyleInfo(style_info) for style_info in payload["style_infos"]
        ]



# voicevox - types/speaker_info

from typing import TypedDict, List


class StyleInfoType(TypedDict):
    id: int
    icon: str
    portrait: str
    voice_samples: List[str]


class SpeakerInfoType(TypedDict):
    policy: str
    portrait: str
    style_infos: List[StyleInfoType]



# voicevox - types/speakers

from typing import TypedDict, List


class StyleType(TypedDict):
    name: str
    id: int


class SupportedFeatureType(TypedDict):
    permitted_synthesis_morphing: str


class SpeakerType(TypedDict):
    supported_features: SupportedFeatureType
    name: str
    speaker_uuid: str
    styles: List[StyleType]
    version: str



# voicevox - http

from typing import List, Optional, Dict

import logging

from httpx import AsyncClient


from .errors import NotfoundError, HttpException
from .types import AudioQueryType, SpeakerType
from .types.speaker_info import SpeakerInfoType 


logger = logging.getLogger(__name__)


class HttpClient:

    def __init__(self, base_url: str, timeout: Optional[int] = None):
        self.session = AsyncClient(base_url=base_url, timeout=timeout)
        logger.debug("Start session.")

    async def close(self) -> None:
        logger.debug("Stop session")
        await self.session.aclose()

    async def request(self, method: str, path: str, **kwargs) -> dict:
        logger.debug(f"Request: {method} Path: {path} kwargs: {kwargs}")
        response = await self.session.request(method, path, **kwargs)
        logger.debug("StatusCode: {0.status_code} Response: {0.content}".format(response))
        if response.status_code == 200 or \
        response.status_code == 204:
            if response.headers.get("content-type") == "application/json":
                return response.json()
            else:
                return response.content
        elif response.status_code == 404:
            raise NotfoundError(response.json()["detail"])
        else:
            raise HttpException(response.json())

    async def synthesis(self, params: dict, payload: dict) -> bytes:
        return await self.request(
            "POST", "/synthesis", params=params,
            json=payload
        )

    async def multi_synthesis(
        self, params: dict, payload: List[dict]
    ) -> bytes:
        return await self.request(
            "POST", "/multi_synthesis", params=params,
            json=payload
        )

    async def create_audio_query(self, params: dict) -> AudioQueryType:
        return await self.request(
            "POST", "/audio_query", params=params
        )

    async def create_audio_query_from_preset(
        self, params: dict
    ) -> AudioQueryType:
        return await self.request(
            "POST", "/audio_query_from_preset", params=params
        )

    async def get_version(self) -> str:
        return await self.request(
            "GET", "/version"
        )

    async def get_core_versions(self) -> List[str]:
        return await self.request("GET", "/core_versions")

    async def get_speakers(
        self, core_version: Optional[str]
    ) -> List[SpeakerType]:
        params = {}
        if core_version is not None:
            params["core_version"] = core_version
        return await self.request("GET", "/speakers", params=params)

    async def get_speaker_info(
        self, speaker_uuid: str, core_version: Optional[str]
    ) -> SpeakerInfoType:
        params = {
            "speaker_uuid": speaker_uuid 
        }
        if core_version is not None:
            params["core_version"] = core_version
        return await self.request("GET", "/speaker_info", params=params)

    async def initialize_speaker(self, params: dict) -> None:
        await self.request("POST", "/initialize_speaker", params=params)

    async def is_initialized_speaker(self, params: dict) -> bool:
        return await self.request("GET", "/is_initialized_speaker", params=params)

    async def supported_devices(self, params: dict) -> Dict[str, bool]:
        return await self.request("GET", "/supported_devices")



"""
.. include:: ../README.rst
"""

from .client import Client
from .audio_query import AudioQuery
from .speakers import Speaker, Style, SupportedFeature

from .errors import HttpException, NotfoundError


__all__ = (
    "Client",
    "AudioQuery",
    "HttpException", "NotfoundError",
    "Speaker", "Style", "SupportedFeature"
)
__version__ = "0.2.0"



# voicevox - types/audio_query

from typing import TypedDict, List


class MoraType(TypedDict):
    text: str
    consonant: str
    consonant_length: int
    vowel: str
    vowel_length: int
    pitch: float


class AccentPhraseType(TypedDict):
    moras: List[MoraType]
    accent: int
    is_interrogative: bool
    pause_mora: List[MoraType]


class AudioQueryType(TypedDict):
    accent_phrases: List[AccentPhraseType]
    speedScale: float
    pitchScale: float
    intonationScale: float
    volumeScale: float
    prePhonemeLength: float
    postPhonemeLength: float
    outputSamplingRate: int
    outputStereo: bool
    kana: str



# voicevox - types

from .audio_query import AudioQueryType, AccentPhraseType, MoraType
from .speakers import SpeakerType, StyleType, SupportedFeatureType


__all__ = (
    "AudioQueryType", "AccentPhraseType", "MoraType",
    "SpeakerType", "StyleType", "SupportedFeatureType"
)



from setuptools import setup
import re


def get_version():
    with open("voicevox/__init__.py", "r") as f:
        return re.search(r'__version__ = "([^"]+)"', f.read()).group(1)


with open("README.rst", "r",  encoding='utf-8') as f:
    long_description = f.read()


with open("requirements.txt", "r") as f:
    requirements = f.readlines()


setup(
    name="voicevox-client",
    description="Voicevox engine unoffical wrapper",
    url="https://github.com/voicevox-client/python",
    project_urls={
        "Documentation": "https://voicevox-client.tuna2134.jp",
        "Source": "https://github.com/voicevox-client/python",
    },
    version=get_version(),
    long_description=long_description,
    long_description_content_type="text/x-rst",
    author="tuna2134",
    license="MIT",
    packages=["voicevox", "voicevox.types"],
    package_data={
        "voicevox": ["py.typed"]
    },
    install_requires=requirements,
    extras_require={
        "tests": [
            "pytest",
            "pytest-asyncio"
        ],
        "docs": [
            "pdoc3"
        ]
    }
)



# voicevox - Client

from typing import Optional, List
from typing_extensions import Self

import logging

from .audio_query import AudioQuery
from .http import HttpClient
from .speakers import Speaker
from .speaker_info import SpeakerInfo
from .supported_devices import SupportedDevices

logger = logging.getLogger(__name__)


class Client:
    """Voicevox client class

    Wrap voicevox engine api.

    Parameters
    ----------
    base_url: str
        Voicevox engine endpoint uri.

    Attributes
    ----------
    http: HttpClient
        Http client attribute.
    timeout: Optional[int]
        You can customize timeout. If you use cpu mode, I recommend to use this.
    """

    def __init__(self, base_url: str = "http://localhost:50021", timeout: Optional[int] = None):
        self.http = HttpClient(base_url=base_url, timeout=timeout)

    async def close(self) -> None:
        """Close http client

        You must run this function, when you finish process.
        """
        logger.info("Close http client")
        await self.http.close()

    async def __aenter__(self) -> Self:
        return self

    async def __aexit__(self, *args) -> None:
        await self.close()

    async def create_audio_query(
        self, text: int, speaker: int, *, core_version: Optional[str] = None
    ) -> AudioQuery:
        """Create audio query

        If you want do tts, you must run first.

        Parameters
        ----------
        text: str
            Text message
        speaker: int
            Speaker id
        core_version: str
            Core version

        Returns
        -------
        AudioQuery
            Audio query, that run synthesis.
        """
        params = {
            "text": text,
            "speaker": speaker
        }
        if core_version is not None:
            params["core_version"] = core_version
        audio_query = await self.http.create_audio_query(params)
        return AudioQuery(self.http, audio_query)

    async def create_audio_query_from_preset(
        self, text: str, preset_id: int, *, core_version: Optional[str] = None
    ) -> AudioQuery:
        params = {
            "text": text,
            "preset_id": preset_id
        }
        if core_version is not None:
            params["core_version"] = core_version
        audio_query = await self.http.create_audio_query_from_preset(params)
        return AudioQuery(self, audio_query, preset_id)

    async def fetch_engine_version(self) -> str:
        """Fetch engine version

        This can fetch voicevox engine version.

        Returns
        -------
        str
            Engine version
        """
        return await self.http.get_version()

    async def fetch_core_versions(self) -> List[str]:
        """Fetch core versions

        This can fetch voicevox core versions.

        Returns
        -------
        List[str]
            Core versions
        """
        return await self.http.get_core_versions()

    async def fetch_speakers(
        self, core_version: Optional[str] = None
    ) -> List[Speaker]:
        """Fetch speakers

        This can fetch voicevox speakers.

        Returns
        -------
        List[Speaker]
            Speakers
        """
        speakers = await self.http.get_speakers(core_version)
        return [Speaker(speaker) for speaker in speakers]
    
    async def fetch_speaker_info(
        self, speaker_uuid: str, core_version: Optional[str] = None
    ) -> SpeakerInfo:
        """Fetch speaker's info by given uuid.

        This function retrieves additional information about a specific speaker, including its voice samples, icon, and portrait images.

        Parameters
        ----------
        speaker_uuid : str
            speaker's uuid
        core_version : Optional[str]
            voicevox core version

        Returns
        -------
        SpeakerInfo
            Contains additional information of the speaker.
        """
        return SpeakerInfo(await self.http.get_speaker_info(speaker_uuid, core_version)) 

    async def check_devices(self, core_version: Optional[str] = None) -> SupportedDevices:
        params = {}
        if core_version:
            params["core_version"] = core_version
        return SupportedDevices(await self.http.supported_devices(params))

    async def multi_synthesis(
        self, audio_queries: List[AudioQuery], speaker: int,
        *, core_version: Optional[str] = None
    ) -> bytes:
        """Multi synthe

        This function is like AudioQuery.synthesis, but it can synthesis multi!

        Parameters
        ----------
        audio_queries: List[AudioQuery]
            Array of audio query
        speaker: int
            speaker id
        core_version: Optional[str]
            voicevox core version

        Returns
        -------
        bytes
            Return zip file"""
        params = {
            "speaker": speaker
        }
        if core_version is not None:
            params["core_version"] = core_version
        return await self.http.multi_synthesis(
            params, [
                audio_query.to_dict()
                for audio_query in audio_queries
            ]
        )

    async def init_speaker(
        self, speaker: int, *, skip_reinit: bool = False,
        core_version: Optional[str] = None
    ) -> None:
        """Initilize speaker

        Initializes the speaker with the specified speaker_id.
        Other APIs can be used without executing this function,
        but it may take some time when it is executed for the first time.

        Parameters
        ----------
        speaker: int
            speaker id
        skip_reinit: bool
            Whether to skip reinitialization of speakers
            who have already been initialized
        core_version: Optional[str]
            core version"""
        params = {
            "speaker": speaker,
            "skip_reinit": skip_reinit
        }
        if core_version is not None:
            params["core_version"] = core_version
        await self.http.initialize_speaker(params)

    async def check_inited_speaker(self, speaker: int, *, core_version: Optional[str] = None):
        """Check initialized speaker

        Returns whether the speaker with the specified speaker_id is initialized or not.

        Parameters
        ----------
        speaker: int
            speaker id
        core_version: Optional[str]
            core version

        Returns
        -------
        bool
            If initialized speaker, it return `True`."""
        params = {
            "speaker": speaker
        }
        if core_version is not None:
            params["core_version"] = core_version
        return await self.http.is_initialized_speaker(params)
